import {useEffect, useRef, useState} from 'react'
import {ID, QUERIES} from '../../../../../../../_metronic/helpers' 
import {StepperComponent} from '../../../../../../../_metronic/assets/ts/components'
import {Formik, Form, FormikValues, Field, ErrorMessage} from 'formik'
import {useQuery, useQueryClient} from 'react-query'
import { createPositionSchemas } from './CreatePositionWizardHelper'
import {getOrganisations, getPositionTypes, savePosition} from '../../core/_requests'
import { useLocation, useParams } from "react-router-dom";
import { number, string } from 'yup'
import { useQueryResponse as useQueryResponsePosition } from '../../core/QueryResponseProviderPosition'
import { useQueryResponse as  useQueryResponseOrganisation } from '../../core/QueryResponseProviderOrganisation' 

import {useFormik, FormikProvider} from 'formik'
import { initialOrganisation, initialPerson, initialPositionType, Position } from '../../core/_models'
import { useListView  } from './ListViewProviderPosition'
import { ListLoading } from '../../components/loading/ListLoading'
import {FC} from 'react'



type Props = {
  isPositionLoading: boolean
  position: Position
}

const PositionEditModalForm: FC<Props> = ({position, isPositionLoading}) => {
  const {setItemIdForUpdate} = useListView()
  const {refetch} = useQueryResponsePosition()
  const queryClient = useQueryClient()
  const queryResponsePosition = useQueryResponsePosition()
  const queryResponseOrganisation = useQueryResponseOrganisation()
  const path = useLocation().pathname
  const id = useParams().id

  const [positionForEdit] = useState<Position>({
    positionType: initialPositionType,
    organisation: initialOrganisation,
    persons : []
  })


  const cancel = (withRefresh?: boolean) => {
    if (withRefresh) {
      refetch()
    }
    setItemIdForUpdate(undefined)
    positionForEdit.organisation = initialOrganisation
    positionForEdit.organisation.id = undefined
    console.log(positionForEdit.organisation?.id)
  }

  const setOrganisationId = (organisationId: ID) => {
    if(positionForEdit.organisation != undefined){
      positionForEdit.organisation.id = organisationId
      queryClient.invalidateQueries([`${QUERIES.POSITION_TYPES_LIST}-${queryResponsePosition.query}`])
      console.log(positionForEdit.organisation.id)
    }
  }

  const formik = useFormik({
    initialValues: positionForEdit,
    onSubmit: async (values, {setSubmitting}) => {
      console.log(values)
      values.organisation = initialOrganisation
      values.organisation.id = positionForEdit.organisation?.id
      if(path == "/person/overview/"+id){
       const person = initialPerson
       person.id = Number(id)
       values.persons?.push(person)
      }
      setSubmitting(true)
      try {
 
          await savePosition(values)
          
      } catch (ex) {
        console.error(ex)
      } finally {
        setSubmitting(true)
        cancel(true)
        queryClient.invalidateQueries([`${QUERIES.POSITIONS_LIST}-${queryResponsePosition.query}`])
        queryClient.invalidateQueries([`${QUERIES.ORGANISATIONS_LIST}-${queryResponsePosition.query}`])
        queryClient.invalidateQueries([`${QUERIES.PERSONS_LIST}-${queryResponsePosition.query}`])
      }
    },
  })

  if(path == "/organisation/overview/"+id && positionForEdit?.organisation != undefined){
    positionForEdit.organisation.id = Number(id)
  }

  const {
    data: organisations
  } = useQuery(
    `${QUERIES.POSITIONS_LIST}-${queryResponseOrganisation.query}`,
    () => {
      return getOrganisations("", 1)
    },
    {cacheTime: 0, keepPreviousData: true, refetchOnWindowFocus: false}
  )

  const {
    data: positionTypes,
  } = useQuery(
    `${QUERIES.POSITION_TYPES_LIST}-${queryResponsePosition.query }`,
    
    () => {
      return getPositionTypes(positionForEdit.organisation?.id)
    },
    {cacheTime: 0, keepPreviousData: true, refetchOnWindowFocus: false}
  )
    console.log(organisations?.data)
    console.log(positionForEdit.organisation?.id)
  return (
    <FormikProvider value={formik}>

    <>
      <form id='kt_modal_add_user_form' className='form' onSubmit={formik.handleSubmit} noValidate>
        {/* begin::Scroll */}
        <div
          className='d-flex flex-column scroll-y me-n7 pe-7'
          id='kt_modal_add_user_scroll'
          data-kt-scroll='true'
          data-kt-scroll-activate='{default: false, lg: true}'
          data-kt-scroll-max-height='auto'
          data-kt-scroll-dependencies='#kt_modal_add_user_header'
          data-kt-scroll-wrappers='#kt_modal_add_user_scroll'
          data-kt-scroll-offset='300px'
        >
          {positionForEdit.organisation?.id == undefined && organisations?.data != undefined?  
                <div className='w-100'>        
            <div className='pb-10 pb-lg-15'>
              <h2 className='fw-bolder d-flex align-items-center text-dark'>
                Välj Organisation
              </h2>
            </div>
      
            <div className='fv-row'>
              <div className='row'>
      
              {organisations?.data.map(organisation=> {
                        return(
                      <div className='fv-row' key={organisation.id}>
                        {/* begin::Radio */}
                        <div className='form-check form-check-custom form-check-solid'>
                          {/* begin::Input */}
                          <input
                            className='form-check-input me-3'
                            //{...formik.getFieldProps('organisation.id')}
                            name='organisation.id'
                            type='radio'
                            id='kt_modal_update_role_option_0'
                            onClick={() => (setOrganisationId(organisation.id))}
                            //disabled={formik.isSubmitting || isPositionLoading}
                          />
                          {/* end::Input */}
                        
                          {/* begin::Label */}
                          <label className='form-check-label' htmlFor='kt_modal_update_role_option_0'>
                            <div className='fw-bolder text-gray-800'>{organisation.name}</div>
      
                          </label>
                          {/* end::Label */}
                          
                        </div>
                        {/* end::Radio */}
                        <div className='fv-row mb-7'> </div>
                      </div>
                      
                      )
                    } )}
              </div>
            </div>
          </div>       
          :null}
                  {positionForEdit.organisation?.id != undefined && positionForEdit.positionType?.id == undefined ?
                    <div className='w-100'>
      <div className='pb-8 pb-lg-10'>
      <h2 className='fw-bolder text-dark'>Välj Befattningstyp</h2>
    </div>

    <div className='fv-row'>
      <div className='row'>

      {positionTypes?.data?.map(positionType=> {
                return(
              <div className='fv-row' key={positionType.id}>
                {/* begin::Radio */}
                <div className='form-check form-check-custom form-check-solid'>
                  {/* begin::Input */}
                  <Field
                    className='form-check-input me-3'
                    //{...formik.getFieldProps('organisation.id')}
                    name='positionType.id'
                    type='radio'
                    value={String(positionType.id)}
                    id='kt_modal_update_role_option_0'
                    //disabled={formik.isSubmitting || isFetching}
                  />
                  {/* end::Input */}
                
                  {/* begin::Label */}
                  <label className='form-check-label' htmlFor='kt_modal_update_role_option_0'>
                    <div className='fw-bolder text-gray-800'>{positionType.name}</div>

                  </label>
                  {/* end::Label */}
                  
                </div>
                {/* end::Radio */}
                <div className='fv-row mb-7'> </div>


              </div>
              
              )
            } )}
      </div>   
    </div>
    </div>
                    
          :null}
                  {positionForEdit.organisation?.id == id && positionForEdit.positionType?.id != undefined?
                  <div className='fv-row mb-7'> </div>
          :null}

        </div>
        {/* end::Scroll */}

        {/* begin::Actions */}
        <div className='text-center pt-15'>
          <button
            type='reset'
            onClick={() => cancel()}
            className='btn btn-light me-3'
            data-kt-users-modal-action='cancel'
            disabled={formik.isSubmitting || isPositionLoading}
          >
            Discard
          </button>

          <button
            type='submit'
            className='btn btn-primary'
            data-kt-users-modal-action='submit'
            disabled={isPositionLoading || formik.isSubmitting || !formik.isValid || !formik.touched}
          >
            <span className='indicator-label'>Submit</span>
            {(formik.isSubmitting || isPositionLoading) && (
              <span className='indicator-progress'>
                Please wait...{' '}
                <span className='spinner-border spinner-border-sm align-middle ms-2'></span>
              </span>
            )}
          </button>
        </div>
        {/* end::Actions */}
      </form>
      {(formik.isSubmitting || isPositionLoading) && <ListLoading />}
    </>
    </FormikProvider>

  )
}
  
  

export {PositionEditModalForm}
